<?php


class ControllerExtensionPaymentRbs extends Controller
{
    /**
     * @param $registry
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->load->language('extension/payment/rbs');
        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/extension/payment/rbs.tpl')) {
            $this->have_template = true;
        }
    }

    /**
     * @return mixed
     */
    public function index()
    {
        $data['action'] = $this->url->link('extension/payment/rbs/payment','',true);
        $data['rbs_button_confirm'] = $this->language->get('rbs_button_confirm');
        return $this->get_template('extension/payment/rbs', $data);
    }

    /**
     * @param $template
     * @param $data
     * @return mixed
     */
    private function get_template($template, $data)
    {
        return $this->load->view($template, $data);
    }

    public function payment()
    {

        // for config settings
        $this->initializeRbs();

        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $order_number = (int)$order_info['order_id'];
        $amount = round($order_info['total'] * $order_info['currency_value'], 2) * 100;
//        print_r($order_info);die;

        $return_url = $this->url->link('extension/payment/rbs/comeback');

        $jsonParams_array = array(
            'CMS' => 'Opencart 2.3.x',
            'Module-Version' => 'RBSPayment ' . $this->rbs->version,
        );

        if (!empty($order_info['email'])) {
            $jsonParams_array['email'] = $order_info['email'];
        }
        if (!empty($order_info['telephone'])) {
            $jsonParams_array['phone'] = "+" . preg_match('/[7]\d{9}/', $order_info['telephone']) ? $order_info['telephone'] : '';
        }

        // here we will collect data for orderBundle
        $orderBundle = [];

        $orderBundle['customerDetails'] = array(
            'email' => $order_info['email'],
            'phone' => preg_match('/[7]\d{9}/', $order_info['telephone']) ? $order_info['telephone'] : ''
        );

        // ITEMS
        foreach ($this->cart->getProducts() as $product) {

            $product_taxSum = $this->tax->getTax($product['price'], $product['tax_class_id']);
            $product_amount = ($product['price'] + $product_taxSum) * $product['quantity'];

            $tax_type = $this->config->get('rbs_taxType');

            // DEFINE TAX_TYPE
            if ($product['tax_class_id'] != 0) {
                $item_rate = $product_taxSum / $product['price'] * 100;
                switch ($item_rate) {
                    case 20:
                        $tax_type = 6;
                        break;
                    case 18:
                        $tax_type = 3;
                        break;
                    case 10:
                        $tax_type = 2;
                        break;
                    case 0:
                        $tax_type = 1;
                        break;
                    default:
                        $tax_type = $this->config->get('rbs_taxType');
                }
            }

            $product_data = array(
                'positionId' => $product['cart_id'],
                'name' => $product['name'],
                'quantity' => array(
                    'value' => $product['quantity'],
                    'measure' => (!empty($item['arrGate']['quantity']['measure'])) ? $item['arrGate']['quantity']['measure'] : $this->rbs->getDefaultMeasurement(),
                ),
                'itemAmount' => $product_amount * 100,
                'itemCode' => $product['product_id'] . "_" . $product['cart_id'], //fix by PLUG-1740, PLUG-2620
                'tax' => array(
                    // todo: some question taxType
                    'taxType' => $tax_type,
                    'taxSum' => $product_taxSum * 100
                ),
                'itemPrice' => ($product['price'] + $product_taxSum) * 100,
            );

            // FFD 1.05 added
//            if ($this->rbs->getFFDVersion() == 'v105') {

            $attributes = array();
            $attributes[] = array(
                "name" => "paymentMethod",
                "value" => $this->rbs->getPaymentMethodType()
            );
            $attributes[] = array(
                "name" => "paymentObject",
                "value" => $this->rbs->getPaymentObjectType()
            );

            $product_data['itemAttributes']['attributes'] = $attributes;
//            }

            $orderBundle['cartItems']['items'][] = $product_data;
        }


        // DELIVERY
        if (isset($this->session->data['shipping_method']['cost']) && $this->session->data['shipping_method']['cost'] > 0) {

            $delivery['positionId'] = 'delivery';
            $delivery['name'] = $this->session->data['shipping_method']['title'];
            $delivery['itemAmount'] = $this->session->data['shipping_method']['cost'] * 100;
            $delivery['quantity']['value'] = 1;
            //todo fix piece
            $delivery['quantity']['measure'] = $this->rbs->getDefaultMeasurement();
            $delivery['itemCode'] = $this->session->data['shipping_method']['code'];
            $delivery['tax']['taxType'] = $this->config->get('rbs_taxType');
            $delivery['tax']['taxSum'] = 0;
            $delivery['itemPrice'] = $this->session->data['shipping_method']['cost'] * 100;

            // FFD 1.05 added
//            if ($this->rbs->getFFDVersion() == 'v105') {

                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $this->rbs->getPaymentMethodType(true)
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 4
                );

                $delivery['itemAttributes']['attributes'] = $attributes;
//            }

            $orderBundle['cartItems']['items'][] = $delivery;
        }


        // VOUCHERS
        if(isset($this->session->data['vouchers']) && count($this->session->data['vouchers']) > 0) {
            foreach ($this->session->data['vouchers'] as $key => $voucher) {

                $itemVoucher = array(
                    'positionId' => 'voucher_' . $key,
                    'name' => $voucher['description'],
                    'itemAmount' => $voucher['amount'] * 100,
                    'quantity' => array(
                        'value' => 1,
                        'measure' => $this->rbs->getDefaultMeasurement(),
                    ),
                    'itemCode' => 'voucher_' . $key,
                    'tax' => array(
                        'taxType' => $this->config->get('rbs_taxType'),
                        'taxSum' => 0,
                    ),
                    'itemPrice' => $voucher['amount'] * 100
                );

                $attributes = array();
                $attributes[] = array(
                    "name" => "paymentMethod",
                    "value" => $this->rbs->getPaymentMethodType(),
                );
                $attributes[] = array(
                    "name" => "paymentObject",
                    "value" => 1
                );

                $itemVoucher['itemAttributes']['attributes'] = $attributes;
                $orderBundle['cartItems']['items'][] = $itemVoucher;
            }
        }


        // DISCOUNT CALCULATE
        $discount = $this->rbs->discountHelper->discoverDiscount($amount, $orderBundle['cartItems']['items']);
        if ($discount > 0) {
            $this->rbs->discountHelper->setOrderDiscount($discount);
            $recalculatedPositions = $this->rbs->discountHelper->normalizeItems($orderBundle['cartItems']['items']);
            $recalculatedAmount = $this->rbs->discountHelper->getResultAmount();
            $orderBundle['cartItems']['items'] = $recalculatedPositions;
        }

        $args = array(
            'orderNumber' => $order_number . "_" . time(),
            'amount' => $amount,
            'currency' => $this->rbs->currency_code2num[$order_info['currency_code']],
            'language' => $this->rbs->language,
            'returnUrl' => $return_url,
            'jsonParams' => json_encode($jsonParams_array),
        );

        if ($this->rbs->rbs_enable_fiscale_options && $this->rbs->ofd_status && !empty($orderBundle)) {
            $args['taxSystem'] = $this->rbs->taxSystem;
            $args['orderBundle']['orderCreationDate'] = date('c');
            $args['orderBundle'] = json_encode($orderBundle);
        }

        //$response = $this->rbs->register_order($order_number, $amount, $return_url, $currency_code, $orderBundle);
        $response = $this->rbs->gateway($this->rbs->stage == 'two' ? 'registerPreAuth.do' : 'register.do', $args);


        if (isset($response['orderId'])) {
            $comment = "orderId: " . $response['orderId'];
            $this->model_checkout_order->addOrderHistory($order_number, 1, "orderId: " .$response['orderId'], true);
        }

        if (isset($response['errorCode'])) {
            $this->document->setTitle($this->language->get('error_title'));

            $data['header'] = $this->load->controller('common/header');
            $data['column_left'] = $this->load->controller('common/column_left');
            $data['column_right'] = $this->load->controller('common/column_right');
            $data['content_top'] = $this->load->controller('common/content_top');
            $data['button_continue'] = $this->language->get('error_continue');

            $data['heading_title'] = $this->language->get('error_title') . ' #' . $response['errorCode'];
            $data['text_error'] = $response['errorMessage'];
            $data['continue'] = $this->url->link('checkout/cart');

            $data['content_bottom'] = $this->load->controller('common/content_bottom');
            $data['footer'] = $this->load->controller('common/footer');

            $this->response->setOutput($this->get_template('error/rbs', $data));
        } else {
            $this->response->redirect($response['formUrl']);
        }
    }

    private function initializeRbs()
    {
        $this->library('rbs');
        $this->rbs = new RBS();
        $this->rbs->login = $this->config->get('rbs_merchantLogin');
        $this->rbs->password = $this->config->get('rbs_merchantPassword');
        $this->rbs->stage = $this->config->get('rbs_stage');
        $this->rbs->mode = $this->config->get('rbs_mode');
        $this->rbs->logging = $this->config->get('rbs_logging');
        $this->rbs->currency = $this->config->get('rbs_currency');
        $this->rbs->taxSystem = $this->config->get('rbs_taxSystem');
        $this->rbs->taxType = $this->config->get('rbs_taxSystem');
        $this->rbs->ofd_status = $this->config->get('rbs_ofd_status');

        $this->rbs->FFDVersion = $this->config->get('rbs_FFDVersion');
        $this->rbs->paymentMethodType = $this->config->get('rbs_paymentMethodType');
        $this->rbs->deliveryPaymentMethodType = $this->config->get('rbs_deliveryPaymentMethodType');
        $this->rbs->paymentObjectType = $this->config->get('rbs_paymentObjectType');

        $c_locale = substr($this->language->get('code'), 0, 2);
        $this->rbs->language = ($c_locale == "ru" || $c_locale == "en") ? $c_locale : "ru";
    }

    /**

     * @param $library
     */
    private function library($library)
    {
        $file = DIR_SYSTEM . 'library/' . str_replace('../', '', (string)$library) . '.php';

        if (file_exists($file)) {
            include_once($file);
        } else {
            trigger_error('Error: Could not load library ' . $file . '!');
            exit();
        }
    }

    // /index.php?route=extension/payment/rbs/callback&mdOrder=1f4c0b12-8523-703d-ba1d-083000abd060
    public function callback()
    {
        if (!$this->rbs->allowCallbacks) {
            exit;
        }

        if (isset($this->request->get['mdOrder'])) {
            $order_id = $this->request->get['mdOrder'];
        } else {
            die('Illegal Access');
        }

        $this->initializeRbs();
        $response = $this->rbs->get_order_status($order_id);

        $ex = explode("_", $response['orderNumber']);
        $order_number = $ex[0];

        $this->load->model('checkout/order');
//        $order_number = $this->session->data['order_id'];
        $order_info = $this->model_checkout_order->getOrder($order_number);

        if ($order_info) {
            if (($response['errorCode'] == 0) && (($response['orderStatus'] == 1) || ($response['orderStatus'] == 2))) {
                // set order status
                $this->model_checkout_order->addOrderHistory($order_number, $this->config->get('rbs_order_status_completed_id'));

//                $this->response->redirect($this->url->link('checkout/success', '', true));
            } else {
//                $this->model_checkout_order->addOrderHistory($order_number, 1);
//                $this->response->redirect($this->url->link('checkout/failure', '', true));
            }
        }
    }

    // /index.php?route=extension/payment/rbs/comeback&orderId=1f4c0b12-8523-703d-ba1d-083000abd060&lang=es
    public function comeback()
    {
        if (isset($this->request->get['orderId'])) {
            $order_id = $this->request->get['orderId'];
        } else {
            die('Illegal Access');
        }

        $this->initializeRbs();
        $response = $this->rbs->get_order_status($order_id);

        $ex = explode("_", $response['orderNumber']);
        $order_number = $ex[0];


        $this->load->model('checkout/order');
        $order_info = $this->model_checkout_order->getOrder($order_number);

        if ($order_info) {

            if (($response['errorCode'] == 0) && (($response['orderStatus'] == 1) || ($response['orderStatus'] == 2))) {
                if (!$this->rbs->allowCallbacks) {
                    // set order status
                    $this->model_checkout_order->addOrderHistory($order_number, $this->config->get('rbs_order_status_completed_id'));
                }
                $this->response->redirect($this->url->link('checkout/success', '', true));
            } else {
                $this->response->redirect($this->url->link('checkout/failure', '', true));
            }
        }
    }

}