<?php
define('RBS_PROD_URL' , 'https://securepayments.sberbank.ru/payment/rest/');
define('RBS_TEST_URL' , 'https://3dsec.sberbank.ru/payment/rest/');
define('RBS_PROD_URL_ALT' , 'https://payment.alfabank.ru/payment/rest/');
define('RBS_PROD_URL_ALT_PREFIX' , 'r-');

define('RBS_ENABLE_LOGGING', true);
define('RBS_ENABLE_FISCALE_OPTIONS', true);
define('RBS_MEASUREMENT_NAME', 'шт');
define('RBS_ENABLE_CALLBACK', true);
?>
<?php


class RBS
{
    public $prod_url;
    public $test_url;
    public $rbs_enable_fiscale_options = RBS_ENABLE_FISCALE_OPTIONS;
    public $logging = RBS_ENABLE_LOGGING;

    public $currency_code2num = array(
        'USD' => '840',
        'UAH' => '980',
        'RUB' => '810',
        'RON' => '946',
        'KZT' => '398',
        'KGS' => '417',
        'JPY' => '392',
        'GBR' => '826',
        'EUR' => '978',
        'CNY' => '156',
        'BYR' => '974',
        'BYN' => '933'
    );

    public $taxSystem;
    public $taxType;
    public $discountHelper;

    public $language = 'ru';
    public $defaultMeasurement = RBS_MEASUREMENT_NAME;
    public $version = '2.7.0';

    public $login;
    public $password;
    public $mode;
    public $stage;
    public $currency;
    public $ofd_status;
    public $FFDVersion;
    public $paymentMethodType;
    public $paymentObjectType;
    private $deliveryPaymentMethodType;


    public $allowCallbacks = RBS_ENABLE_CALLBACK;

    public function __construct()
    {

        // Endpoints
        $this->test_url = RBS_TEST_URL;
        $this->prod_url = RBS_PROD_URL;

        // for migrate to new payment gateway
        if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
            //disable callbacks for old gateway version
            $this->allowCallbacks = false;
        }

        if (RBS_ENABLE_FISCALE_OPTIONS === true){
            $this->library('rbs_discount');
            $this->discountHelper = new rbsDiscount();
        }
    }

    private function library($library)
    {
        $file = DIR_SYSTEM . 'library/' . str_replace('../', '', (string)$library) . '.php';

        if (file_exists($file)) {
            include_once($file);
        } else {
            trigger_error('Error: Could not load library ' . $file . '!');
            exit();
        }
    }

    /**
     * @return mixed
     */
    public function getFFDVersion()
    {
        return $this->FFDVersion;
    }

    /**
     * @param $delivery
     * @return mixed
     */
    public function getPaymentMethodType($delivery = false)
    {
        if ($delivery) {
            return $this->deliveryPaymentMethodType;
        }

        return $this->paymentMethodType;
    }

    /**
     * @return mixed
     */
    public function getPaymentObjectType()
    {
        return $this->paymentObjectType;
    }

    /**
     * @return string
     */
    public function getDefaultMeasurement()
    {
        return $this->defaultMeasurement;
    }

    /**
     *
     * @param $property
     * @param $value
     * @return $this
     */
    public function __set($property, $value)
    {
        if (property_exists($this, $property)) {
            $this->$property = $value;
        }
        return $this;
    }

    /**
     *
     * @param string $order_number
     * @param integer $amount
     * @param integer $currency_code
     * @param string $return_url
     * @param null $orderBundle
     * @return mixed[]
     */
    public function register_order($order_number, $amount, $return_url, $currency_code = 643, $orderBundle = null)
    {

        $data = array(
            'orderNumber' => $order_number . "_" . time(),
            'amount' => $amount,
            'returnUrl' => $return_url,
            'jsonParams' => json_encode(
                [
                    'CMS:' => 'Opencart 2.3.x',
                    'Module-Version: ' => $this->version
                ]
            ),
        );

        // fix only PLUG-2251
        //ini_set('serialize_precision', 14);
        //ini_set('precision', 14);

        if ($this->currency != 0) {
            $data['currency'] = $this->currency;
        } else {
            $data['currency'] = $currency_code;
        }

        if ($this->ofd_status && !empty($orderBundle)) {
            $data['taxSystem'] = $this->taxSystem;

            $data['orderBundle']['orderCreationDate'] = date('c');
            $data['orderBundle'] = json_encode($orderBundle);
        }


        return $this->gateway($this->stage == 'two' ? 'registerPreAuth.do' : 'register.do', $data);
    }

    /**
     *
     * @param string $method
     * @param mixed[] $data
     * @return mixed[]
     */
    public function gateway($method, $data) {

        $data['userName'] = $this->login;
        $data['password'] = $this->password;
        $data['language'] = $this->language;

        if ($this->mode == 'test') {
            $url = $this->test_url;
        } else {
            $url = $this->prod_url;

            // for migrate to new payment gateway
            if (defined('RBS_PROD_URL_ALT') && defined('RBS_PROD_URL_ALT_PREFIX')) {
                if (substr($this->login, 0, strlen(RBS_PROD_URL_ALT_PREFIX)) == RBS_PROD_URL_ALT_PREFIX) {
                    $url = RBS_PROD_URL_ALT;
                }
            }
        }

        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_VERBOSE => true,
            CURLOPT_SSL_VERIFYHOST => false,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_URL => $url.$method,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => http_build_query($data, '', '&'),
            CURLOPT_HTTPHEADER => array('CMS: OpenCart 2.3.x', 'Module-Version: ' . $this->version),
        ));

        $response = curl_exec($curl);
        if ($this->logging) {
            $this->logger($url, $method, $data, $response);
        }
        $response = json_decode($response, true);
        curl_close($curl);

        return $response;
    }

    /**
     *
     * @param string $url
     * @param string $method
     * @param mixed[] $request
     * @param mixed[] $response
     * @return integer
     */
    public function logger($url, $method, $request, $response) {
        $this->library('log');
        $file_name = "oc23x_rbspayment_" . date("y-m") . ".log";
        $logger = new Log($file_name);
        $logger->write("RBS PAYMENT: ".$url.$method."\nREQUEST: ".json_encode($request). "\nRESPONSE: ".$response."\n\n");
    }


    /**
     *
     * @param string $orderId
     * @return mixed[]
     */
    public function get_order_status($orderId)
    {
        return $this->gateway('getOrderStatusExtended.do', array('orderId' => $orderId));
    }

}